/* antcn.cc
 *
 * When        Who  What
 * 1995-11-01  amn  Cleaning up extra log messages.
 * 1995-10-31  amn  Parallel port I/O.
 * 1995-10-29  amn  Initial testing.
 * 1995-10-25  amn  Original Metsahovi version.
 *
 * This was the stub version of antcn (ANTenna CoNtrol program) of fs-9.0.7.
 */

/* Input */
/* IP(1) = mode
       0 = initialize LU
       1 = pointing (from SOURCE command)
       2 = offset (from RADECOFF, AZELOFF, or XYOFF commands)
       3 = on/off source status (from ONSOURCE command)
       4 = direct communications (from ANTENNA command)
       5 = on/off source status for pointing programs
       6 = reserved for future focus control
       7 = log tracking data (from TRACK command)
  8 - 99 = reserved for future use
100 - 32767 = for site specific use

   IP(2) = class number (mode 4 only)
   IP(3) = number of records in class (mode 4 only)
   IP(4) - not used
   IP(5) - not used
*/

/* Output */
/*  IP(1) = class with returned message
      (2) = number of records in class
      (3) = error number
            0 - ok
           -1 - illegal mode
           -2 - timeout
           -3 - wrong number of characters in response
           -4 - interface not set to remote
           -5 - error return from antenna
           -6 - error in pointing model initialization
            others as defined locally
      (4) = 2HAN for above errors, found in FSERR.CTL
          = 2HST for site defined errors, found in STERR.CTL
      (5) = not used
*/

/* Defined variables */
#define MINMODE 0  /* min,max modes for our operation */
#define MAXMODE 7

#define PORT_NO 1  /* '/dev/lp[012]' to use for parallel port rx control */

/* Include files */
#include <stdio.h>
#include <string.h>

#include "antmh.hh"
#include "antmhpar.hh"

extern "C" {
#define class iClass
#include "../../fs/include/params.h" /* FS parameters            */
#include "../../fs/include/fs_types.h" /* FS header files        */
#include "../../fs/include/fscom.h"  /* FS shared mem. structure */
#include "../../fs/include/shm_addr.h" /* FS shared mem. pointer */
#undef class
};

struct fscom *fs;

/* Ordinary C subroutines called */
extern "C" {
void setup_ids();
void putpname(char *);
void skd_wait(char pname[5], long ip[5], unsigned par);
void skd_run(), cls_clr();
int nsem_test();
void logit(char *, int, char *);
int cls_rcv(long iClass, char *buf, int iblen,
	    int *r1, int *r2, int p1, int p2);
void cls_snd(long *iClass, char *buf, int iblen, int p1, int p2);
};


/* antcn main program starts here */
main()
{
  int ierr, nrec, nrecr;
  int dum = 0;
  int r1, r2;
  int imode,i,nchar;
  long ip[5], iClass, clasr;
  char buf[80], buf2[100];

/* Set up IDs for shared memory, then assign the pointer to
   "fs", for readability.
*/
  setup_ids();
  fs = shm_addr;

/* Put our program name where logit can find it. */

  putpname("antcn");

/* Return to this point to wait until we are called again */

Continue:
  skd_wait("antcn",ip,(unsigned)0);

  imode = ip[0];
  iClass = ip[1];
  nrec = ip[2];

  if (imode < MINMODE || imode > MAXMODE) {
    ierr = -1;
    goto End;
  }

/* Handle each mode in a separate section */

  ierr = 0;
  nrecr = 0;
  clasr = 0;
  switch (imode) {

    case 0:             /* initialize */
#if DEBUG
      strcpy(buf,"Initializing antenna interface");
      logit(buf,0,NULL);
#endif
      {  /* Beware, non-null terminated Hollerith string in fscom: */
        char *end;
        char ant_dev[255];

        end = (char *)memccpy(ant_dev, fs->idevant, ' ', sizeof(ant_dev)-1);
        if (end) {
          *(end-1) = '\0';
        } else {
          ant_dev[sizeof(ant_dev)-1] = '\0';
        }
        fsAntOpen(ant_dev);
      }

      (void)parOpen(PORT_NO);

      fs->ionsor = 0;
      break;

    case 1:             /* source= command */
#if DEBUG
      strcpy(buf,"Commanding to a new source");
      logit(buf,0,NULL);
#endif
/* If 'fs->lsorna' is:
     azel          true corrected coordinates
     azeluncr      encoder units
     xy            encoder units
     stow          Pre-defined stow position.
     service       Pre-defined service position.
     disable       Stops antenna communications. This is useful to stop error
                   messages when here is a problem.
     idle          Monitor the antenna position, but do not command it. This is
                   the normal start-up state for the Field System.
     hold          Maintain the current antenna position.
*/
      if (memcmp(fs->lsorna, "idle    ", 8) == 0) {
        fsGotoIdle();
      } else {
	if (fsStartTracking(fs->lsorna, fs->ra50, fs->dec50, fs->ep1950,
			    fs->radat, fs->decdat, fs->epoch)
	    ) {
	  strcpy(buf,"NAK");
	} else {
	  strcpy(buf,"ACK");
	}
	logit(buf,0,NULL);
      }  /* else start tracking */
      fs->ionsor = 0;
      break;

    case 2:             /* offsets         */
#if DEBUG
      sprintf(buf,"Commanding new offsets: az(rad)==%f, el(rad)==%f", fs->AZOFF, fs->ELOFF);
      logit(buf,0,NULL);
#endif
      fsUpdateOffsets(fs->RAOFF, fs->DECOFF,
		      fs->AZOFF, fs->ELOFF,
		      fs->XOFF, fs->YOFF);
      fs->ionsor = 0;
      break;

    case 3:        /* onsource command with error message */
#if DEBUG
      strcpy(buf,"Checking onsource status, extended error logging");
      logit(buf,0,NULL);
#endif
      fs->ionsor = fsOnsource(buf, NULL);
      logit(buf,0,NULL);
      break;

    case 4:            /* direct antenna= command */
      if (iClass == 0)
        goto End;
      for (i=0; i<nrec; i++) {
        nchar = cls_rcv(iClass,buf,sizeof(buf),&r1,&r2,dum,dum);
        buf[nchar] = '\0';  /* make into a string */
#if DEBUG
        strcpy(buf2,"Received message for antenna: ");
        strcat(buf2,buf);
        logit(buf2,0,NULL);
#endif

	/* Process 'buf' command string. */
	/* 'ANTENNA=PAROUT 2 1' == set bit 2 (0..7) to 1 */
	/* 'ANTENNA=PARIN 2 NORXLOCK RXLOCK' == report 'NORXLOCK' */
	/*          when bit 2 is 0. */
        (void)sscanf(buf, "%s ", buf2);
	if (memcmp(buf2, "PAROUT", 6) == 0) {
	  int bitNo, newState;

	  (void)sscanf(buf, "%*s %d %d", &bitNo, &newState);
	  parSet(PORT_NO, bitNo, newState);
#if DEBUG
	  sprintf(buf, "Setting parallel bit %d to %d", bitNo, newState);
	  logit(buf,0,NULL);
#endif
	} else if (memcmp(buf2, "PARIN", 5) == 0) {
	  int bitNo, oldState;

	  (void)sscanf(buf, "%*s %d", &bitNo);
	  oldState = parGet(PORT_NO, bitNo);
	  if (oldState) {
	    (void)sscanf(buf, "%*s %*d %*s %s", buf2);
	  } else {
	    (void)sscanf(buf, "%*s %*d %s %*s", buf2);
	  }
#if DEBUG
	  sprintf(buf, "Parallel bit %d is %d == '%s'", bitNo, oldState, buf2);
#else
	  sprintf(buf, "%s", buf2);
#endif
	  logit(buf,0,NULL);
	} else {  /* no 'ANTENNA=command...' matched */
	  sprintf(buf, "Special antenna command '%s' not implemented.", buf2);
	  logit(buf,0,NULL);
	}

	/* Send successful acknowledge. */
        strcpy(buf,"ACK");
        cls_snd(&clasr,buf,3,dum,dum);
        nrecr += 1;
      }  /* for each comma-separated parameter */
      break;

    case 5:    /* onsource command with no error logging */
#if DEBUG
      strcpy(buf,"Checking onsource status, no error logging");
      logit(buf,0,NULL);
#endif
      fs->ionsor = fsOnsource(NULL, NULL);
      break;

    case 6:            /* reserved */
      ierr = -1;
      goto End;
      break;

    case 7:    /* onsource command with additional info  */
#if DEBUG
      strcpy(buf,"Checking onsource status, log tracking data");
      logit(buf,0,NULL);
#endif
      fs->ionsor = fsOnsource(buf, buf2);
      logit(buf,0,NULL);
      logit(buf2,0,NULL);
/* A sample response from 'track' SNAP command help file:
tr   yy/ddd.hh:mm:ss       cmd1    cmd2    delta1    delta2
pr                         prd1    prd2    praz      prel
co                         cor1    cor2    eqeq      refr
 */
  }  /* end of switch */

End:
  ip[0] = clasr;
  ip[1] = nrecr;
  ip[2] = ierr;
  memcpy(ip+3,"AN",2);
  ip[4] = 0;
  goto Continue;

}
